#
# ProgressBar (pure Ruby/Tk version)
#                2009/11/29  by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#
require 'tk'

module Tk
  module RbWidget
    class ProgressBar < TkWindow
    end
  end
end


class Tk::RbWidget::ProgressBar
  include TkComposite

  def initialize_composite(keys = {})
    # defaults
    bar_color     = 'gray60'
    trough_color  = 'gray90'
    text_color    = 'black'
    reverse_color = 'white'

    font_size = -12

    @width  = 150
    @height =  20

    # create widgets
    @font = TkFont.new(['Helvetica', font_size])

    @var = TkVariable.new('')

    @frame.configure(:borderwidth=>3, :relief=>:ridge)

    @size_base = Tk::Frame.new(@frame, :width=>@width, :height=>@height, 
                               :borderwidth=>0)

    @size_base.pack_propagate(false)

    @base_label = Tk::Label.new(@size_base, :textvariable=>@var, :font=>@font, 
                                :highlightthickness=>0, :borderwidth=>0, 
                                :foreground=>text_color, 
                                :background=>trough_color, 
                                :padx=>1, :pady=>1).pack(:fill=>:both, 
                                                         :expand=>true)
    @prog_bar = Tk::Frame.new(@base_label, 
                              :highlightthickness=>0, :borderwidth=>0
                              ).place(:x=>0, :y=>0, 
                                      :anchor=>:nw, 
                                      :relwidth=>0.0, :relheight=>1.0)

    @prog_label = Tk::Label.new(@prog_bar, :textvariable=>@var, :font=>@font, 
                                :foreground=>reverse_color, 
                                :background=>bar_color,  
                                :highlightthickness=>0, :borderwidth=>0, 
                                :padx=>1, :pady=>1).place(:x=>0, :y=>0, 
                                                          :width=>@width, 
                                                          :height=>@height,
                                                          :anchor=>:nw)

    # for bindings
    @size_base.bindtags  <<= @frame
    @base_label.bindtags <<= @frame
    @prog_bar.bindtags   <<= @frame
    @prog_label.bindtags <<= @frame

    @size_base.bind('Configure'){
      Tk.update_idletasks
      @width = @size_base.winfo_width
      @prog_label.place(:width=>@width)
      @height = @size_base.winfo_height
      @prog_label.place(:height=>@height)
    }

    @size_base.pack(:expand=>true, :fill=>:both)

    # delegates
    delegate('DEFAULT', @frame)
    delegate('background', @prog_label)
    delegate('foreground', @base_label)

    delegate_alias('barcolor',     'background', @prog_label)
    delegate_alias('troughcolor',  'background', @base_label)
    delegate_alias('textcolor',    'foreground', @base_label)
    delegate_alias('reversecolor', 'foreground', @prog_label)

    delegate_alias('fontfamily', 'family', @font)
    delegate_alias('fontsize',   'size',   @font)
    delegate_alias('fontweight', 'weight', @font)
    delegate_alias('fontslant',  'slant',  @font)

    option_methods(:width, :height, :font)

    configure(keys) unless keys.empty?
  end

  def set(rate, text=nil)
    @prog_bar.place(:relwidth=>rate.to_f)
    @var.value = text if text
  end

  def get
    [TkPlace.current_configinfo(@prog_bar, :relwidth)['relwidth'], @var.value]
  end

  def width(size=nil)
    if size
      @size_base.width = size
      @prog_label.place(:width=>size)
      @width = size
    else
      @width
    end
  end

  def height(size=nil)
    if size
      @size_base.height = size
      @prog_label.place(:height=>size)
      @height = size
    else
      @height
    end
  end

  def font(fnt=nil)
    if fnt
      if fnt.kind_of?(TkFont)
        @base_label.font = @prog_label.font = @font = fnt
      elsif fnt.kind_of?(Hash)
        @font.configure(fnt)
      else
        fnt = TkComm.simplelist(fnt) if fnt.kind_of?(String)
        conf = {}
        conf['family'] = fnt.shift if fnt[0]
        conf['size'] = fnt.shift if fnt[0]
        fnt.each{|param|
          case param.to_s
          when 'normal', 'bold'
            conf['weight'] = param
          when 'roman', 'italic'
            conf['slant'] = param
          when 'underline'
            conf['underine'] = true
          when 'overstrike'
            conf['overstrike'] = true
          end
        }

        @font.configure(conf)
      end
    else
      @font
    end
  end
end

if __FILE__ == $0
  TkLabel.new(:text=>"Please click a progress bar.").pack

  bar1 = Tk::RbWidget::ProgressBar.new.pack
  bar2 = Tk::RbWidget::ProgressBar.new.pack(:fill=>:both, :expand=>true)
  bar3 = Tk::RbWidget::ProgressBar.new(:width=>120, :height=>30, 
                                       :barcolor=>'blue', 
                                       :troughcolor=>'white', 
                                       :fontsize=>14, :fontweight=>:bold).pack
  bar4 = Tk::RbWidget::ProgressBar.new(:font=>'courier 8 bold', 
                                       :textcolor=>'red', 
                                       :reversecolor=>'yellow', 
                                       :barcolor=>'gray25', 
                                       :width=>250, :height=>12).pack
  f = TkFrame.new.pack(:fill=>:x)
  bar5 = Tk::RbWidget::ProgressBar.new(f).pack(:side=>:right, :anchor=>:e)
  TkLabel.new(f, :text=>"bar-title:").pack(:side=>:right)

  bar1.set(0.0, '0%')
  bar2.set(0.0, '0%')
  bar3.set(0.0, '0%')
  bar4.set(0.0, '0%')
  bar5.set(0.0, 'rest 100%')

  bar1.bind('1'){puts "bar1 value = #{bar1.get.inspect}"}
  bar2.bind('1'){puts "bar2 value = #{bar2.get.inspect}"}
  bar3.bind('1'){puts "bar3 value = #{bar3.get.inspect}"}
  bar4.bind('1'){puts "bar4 value = #{bar4.get.inspect}"}
  bar5.bind('1'){puts "bar5 value = #{bar5.get.inspect}"}

  timer = TkTimer.new(50, 100){|timerobj|
    cnt = timerobj.return_value + 1
    bar1.set(cnt/100.0, "#{cnt}%")
    bar2.set(cnt/100.0, "#{cnt}%")
    bar3.set(cnt/100.0, "#{cnt}%")
    bar4.set(cnt/100.0, "#{cnt}%")
    bar5.set(cnt/100.0, "rest #{100 - cnt}%")
    cnt
  }

  b = TkButton.new(:text=>'restart', :command=>proc{
                     bar1.set(0.0, '0%')
                     bar2.set(0.0, '0%')
                     bar3.set(0.0, '0%')
                     bar4.set(0.0, '0%')
                     bar5.set(0.0, 'rest 100%')
                     timer.restart{0}
                   }).pack
  b.invoke

  Tk.mainloop
end
